@@ -927,6 +927,12 @@ def save_eig_and_loss_logscaled_instrument_separated(eigenvals, bins1, bins2, tr
 
     return dir_, file
 
+def all_empty(hist):
+    flag = True
+    for i in range(len(hist)):
+        if np.isnan(hist[i]) == False:
+            flag = False
+    return flag
 
 def Histogram_to_Track_OneInstrument_logvelocity3(hists, Instrument_dict, note_length=4, channel=1):
     # Function takes inputs:
@@ -1175,6 +1181,98 @@ def save_SNR_and_loss_logscaled(Data, bin_count, train_loss, val_loss, LambdaSNR
     return dir_, file, histsx, histsy
 
 
+def save_SNR_and_loss_logscaled2(Data, bin_count, train_loss, val_loss, LambdaSNR=True, checkpoints=[], log=False,
+                                 dir_="hist_grad_loss/", file="hist_grad_loss_plot.jpg"):
+    # Data must eigther be the Gammas(scalarproduct of eigenvektors and gradient) or Lambdas(scalarproduct of eigenvectors in Gram space w.r.t GGN)
+    # eigenvectors are conjungate gradients of GGN
+
+    Path(dir_).mkdir(parents=True, exist_ok=True)
+    matplotlib.rcParams.update({'font.size': 20})
+
+    Data_NaN = Replace_0_with_NaN(Data)
+    Data_NaN_mean = np.nanmean(Data_NaN, axis=1)
+    Data_NaN_var = np.nanvar(Data_NaN, axis=1, ddof=1)
+    SNR = Data_NaN_mean ** 2 / Data_NaN_var
+    # print(Data_NaN_mean.shape)
+    # print(SNR.shape)
+
+    xmin, xmax = np.nanmin(Data_NaN_mean), np.nanmax(Data_NaN_mean)
+    ymin, ymax = np.nanmin(SNR), np.nanmax(SNR)
+
+    print("xmin: {}\nxmax: {}".format(xmin, xmax))
+    print("ymin: {}\nymax: {}".format(ymin, ymax))
+
+    binsx = np.logspace(np.log10(xmin), np.log10(xmax), bin_count + 1)
+    binsy = np.logspace(np.log10(ymin), np.log10(ymax), bin_count + 1)
+
+    histsx_ = get_hists_unweighted(Data_NaN_mean, binsx, len(checkpoints))
+    histsy_ = get_hists_unweighted(SNR, binsy, len(checkpoints))
+
+    max_histsx = np.nanmax(histsx_)
+    max_histsy = np.nanmax(histsy_)
+
+    # initialize values for Loss
+    max_train = np.max(train_loss)
+    max_val = np.max(val_loss)
+    max_total = max(max_train, max_val)
+
+    histsx = np.zeros((Data.shape[0], bin_count))
+    histsy = np.zeros((Data.shape[0], bin_count))
+
+    for i in range(Data.shape[0]):
+        # print(i)
+        fig = plt.figure(figsize=(20, 10))
+        # fig = plt.figure(figsize=(10, 10))
+        gs = fig.add_gridspec(1, 2, width_ratios=(7, 5))
+        # width_ratios=(7, 2, 7), height_ratios=(2, 7),
+        #              left=0.1, right=0.9, bottom=0.1, top=0.9,
+        #              wspace=0.05, hspace=0.05)
+        gs0 = gs[0].subgridspec(2, 2, width_ratios=(7, 2), height_ratios=(2, 7),
+                                wspace=0.05, hspace=0.05)
+        gs1 = gs[1].subgridspec(2, 1, height_ratios=(2, 7), wspace=0.05, hspace=0.05)
+
+        ax = fig.add_subplot(gs0[1, 0])
+        ax_histx = fig.add_subplot(gs0[0, 0], sharex=ax)
+        ax_histy = fig.add_subplot(gs0[1, 1], sharey=ax)
+        ax_histx.set_ylim([0, max_histsx])
+        ax_histy.set_xlim([0, max_histsy])
+        # gs[1,2].subgridspec(gs[1,2].get_position()[0],gs[1,2].get_position()[1], wspace = 0.5)
+
+        histsx[i], histsy[i] = scatter_hist(Data_NaN_mean[i], SNR[i], ax, ax_histx, ax_histy, binsx, binsy,
+                                            checkpoints[i], LambdaSNR=LambdaSNR, log=log)
+
+        loss_plot = fig.add_subplot(gs1[1])
+        bins2 = np.linspace(0, i, i + 1)
+        loss_plot.plot(bins2, train_loss[0:i + 1], "-", label="train_loss", linewidth=2.5)
+        loss_plot.plot(bins2, val_loss[0:i + 1], "-", label="val_loss", linewidth=2.5)
+        loss_plot.set_xlim([-1, train_loss.shape[0] + 1])
+        loss_plot.set_ylim([-0.3, max_total + 1])
+        loss_plot.set_title('Training and Validation Loss')
+        loss_plot.set_xlabel("Epoch", fontfamily="serif", fontsize='medium')
+        loss_plot.set_ylabel("Loss", fontfamily="serif", fontsize='medium')
+        plot_loss_axes(checkpoints, loss_plot)
+        loss_plot.legend()
+
+        """
+        # plot loss of current epoch
+        bins2 =  np.linspace(0,i,i+1)
+        ax2.plot(bins2, np.nanmean(train_batch_loss[0:i+1],axis=1),"-",label="train_loss",linewidth=2.5)
+        ax2.plot(bins2, np.nanmean(val_batch_loss[0:i+1],axis=1),"-",label="val_loss",linewidth=2.5)
+        ax2.set_xlim([-1,train_batch_loss.shape[0]+1])
+        ax2.set_ylim([-0.3,max_total+1])
+        ax2.set_title('Training and Validation Loss')
+        ax2.set_xlabel("Epoch",fontfamily ="serif", fontsize ='medium')
+        ax2.set_ylabel("Loss",fontfamily = "serif" , fontsize ='medium')
+        ax2.legend()
+        """
+        plt.savefig(dir_ + str(i) + "_" + file)
+
+        # plt.tight_layout()
+        plt.close()
+
+    return dir_, file, histsx, histsy
+
+
 def save_SNRGamma_and_loss_logscaled(Lambdas, Gammas, bin_count, train_loss, val_loss, LambdaSNR=False, checkpoints=[],
                                      ycutoff=1e-10, log=False, dir_="hist_grad_loss/", file="hist_grad_loss_plot.jpg"):
     # Data must eigther be the Gammas(scalarproduct of eigenvektors and gradient) or Lambdas(scalarproduct of eigenvectors in Gram space w.r.t GGN)
